from . import accounts


class CostClient(object):
    CURRENT_COST_URL = '/olap_reports/cost/current'
    HISTORY_COST_URL = '/olap_reports/cost/history'

    def __init__(self, client):
        self.client = client

    def list_months(self, type=None):
        response = self.client.get(self.HISTORY_COST_URL)

        list_of_months = []
        months = response['dimensions'][0]["time"]

        for month in months:
            label = month['label']
            list_of_months.append(label.encode('ascii'))

        return list_of_months

    def get_current(self, account_name=None, account_type='AWS-Account'):
        response = self.client.get(self.CURRENT_COST_URL)

        accounts_total_cost = []

        accounts_client = accounts.AccountsClient(self.client)
        list_of_aws_accounts = accounts_client.list(account_type)

        costs = response['data']
        for accounts_total in costs:
            accounts_total_cost.append(accounts_total[0][0])

        cost_by_account = dict(zip(list_of_aws_accounts, accounts_total_cost))

        if account_name:
            return cost_by_account[account_name]
        else:
            return cost_by_account['Total']



########
# Copyright (c) 2014 GigaSpaces Technologies Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
#    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    * See the License for the specific language governing permissions and
#    * limitations under the License.

import requests
from requests.packages import urllib3

from .cost import CostClient
from .usage import UsageClient
from .assets import AssetsClient
from .reports import ReportsClient
from .accounts import AccountsClient


DEFAULT_CLOUDHEALTH_API_URL = 'https://chapi.cloudhealthtech.com/'

urllib3.disable_warnings(urllib3.exceptions.InsecurePlatformWarning)


class HTTPClient(object):
    def __init__(self,
                 endpoint,
                 api_key,
                 headers=None,
                 query_params=None,
                 cert=None,
                 trust_all=False):
        self.endpoint = endpoint
        self.api_key = api_key
        self.headers = headers.copy() if headers else {}
        if not self.headers.get('Content-type'):
            self.headers['Content-type'] = 'application/json'

    def get(self,
            uri,
            data=None,
            params=None,
            headers=None,
            _include=None,
            expected_status_code=200,
            stream=False):
        url = self.endpoint + uri + '?api_key={0}'.format(self.api_key)
        response = requests.get(url,
                                data=data,
                                params=params,
                                headers=headers,
                                stream=stream)
        if response.status_code != 200:
            raise RuntimeError(
                'Request to {0} failed! (HTTP Error Code: {1})'.format(
                    url, response.status_code))
        return response.json()


class CloudHealth(object):
    def __init__(self, api_key, endpoint=DEFAULT_CLOUDHEALTH_API_URL):
        self._client = HTTPClient(endpoint, api_key)

        self.accounts = AccountsClient(self._client)
        self.reports = ReportsClient(self._client)
        self.assets = AssetsClient(self._client)
        self.cost = CostClient(self._client)
        self.usage = UsageClient(self._client)



# https://chapi.cloudhealthtech.com/olap_reports?api_key=<your api key>

from cloudhealth import exceptions


class ReportsClient(object):
    def __init__(self, client):
        self.client = client

    def list(self, topic=None):
        reports = []

        uri = '/olap_reports'
        if topic:
            uri = uri + '/{0}'.format(topic)
        response = self.client.get(uri)
        report_links = response['links']
        for _, link_item in report_links.items():
            reports.append(link_item['href'])

        return reports

    def topics(self):
        topics = []

        reports = self.list()
        for report in reports:
            topic = report.split('/')[-1]
            topics.append(topic)

        return topics

    def get(self, id=None, topic=None, report_name=None):
        if id:
            uri = '/olap_reports/custom/{0}'.format(id)
        elif topic and report_name:
            uri = '/olap_reports/{0}/{1}'.format(topic, report_name)
        else:
            raise exceptions.CloudHealthError(
                'Must either provide a report id or a topic and report-name')

        report = self.client.get(uri)
        return report



from datetime import date, timedelta


def _get_yesterdays_date():
    return (date.today() - timedelta(1)).strftime('%Y-%m-%d')


class UsageClient(object):
    USAGE_URL = 'olap_reports/usage'

    def __init__(self, client):
        self.client = client

    def list_days(self, uri):
        response = self.client.get(uri)


        list_of_days = []
        days = response['dimensions'][0]["time"]

        for day in days:
            label = day['label']
            list_of_days.append(label.encode('ascii'))

        return list_of_days

    def get(self, resource_type, date=_get_yesterdays_date()):
        uri = self.USAGE_URL + '/' + resource_type
        response = self.client.get(uri)

        total_usage = []

        list_of_days = self.list_days(uri)

        costs = response['data']
        for days_usage in costs:
            total_usage.append(days_usage[0][0])

        usage_for_day = dict(zip(list_of_days, total_usage))

        return usage_for_day[date]


class AccountsClient(object):
    CURRENT_COST_URL = '/olap_reports/cost/current'

    def __init__(self, client):
        self.client = client

    def list(self, type='AWS-Account'):
        response = self.client.get(self.CURRENT_COST_URL)

        list_of_accounts = []
        accounts_list = response['dimensions'][0][type]

        for account in accounts_list:
            label = account['label']
            list_of_accounts.append(label.encode('ascii'))

        return list_of_accounts



class AssetsClient(object):
    def __init__(self, client):
        self.client = client

    def list(self):
        assets = []

        uri = '/api.json'
        assets = self.client.get(uri)
        return assets

    def get(self, object_name):

        uri = '/api/{0}.json'.format(object_name)
        # if object_name not in self.list():
        #     raise exceptions.CloudHealthError(
        #         'Object {0} does not exist'.format(object_name))

        asset = self.client.get(uri)
        return asset



########
# Copyright (c) 2013 GigaSpaces Technologies Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
#    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    * See the License for the specific language governing permissions and
#    * limitations under the License.

import click

from click_didyoumean import DYMGroup

from cloudhealth import client


CLICK_CONTEXT_SETTINGS = dict(
    help_option_names=['-h', '--help'],
    token_normalize_func=lambda param: param.lower(),
    ignore_unknown_options=True)


@click.group(context_settings=CLICK_CONTEXT_SETTINGS, cls=DYMGroup)
@click.option('--api-key',
              required=True,
              envvar='CLOUDHEALTH_API_KEY',
              help='The API key to your Cloudhealth Account')
@click.pass_context
def _cloudhealth(ctx, api_key):
    """A CloudHealth Command Line Interface

    You can set the `CLOUDHEALTH_API_KEY` environment variable instead
    of using the `--api-key` everytime.
    """
    # TODO: Expose `--api-key` to all commands.
    ctx.obj = {}
    ctx.obj['client'] = client.CloudHealth(api_key)


@_cloudhealth.group(context_settings=CLICK_CONTEXT_SETTINGS, cls=DYMGroup)
@click.pass_context
def cost(ctx):
    """Retrieve cost related information
    """
    ctx.obj['client'] = ctx.obj['client'].cost


@cost.command('current')
@click.option('-n',
              '--account-name',
              help='The account to get the cost for')
@click.option('-t',
              '--account-type',
              default='AWS-Account',
              help='The type to get the cost for [default: AWS-Account]')
@click.pass_context
def current_cost(ctx, account_name, account_type):
    """Retrieve current cost.

    Specifying an account name will get the current cost for that account only.
    Specifying an account type will get the cost to all accounts of that type.
    Omitting both will get the total cost of all accounts.
    """
    cost = ctx.obj['client']
    print(cost.get_current(account_name, account_type))


@_cloudhealth.group(context_settings=CLICK_CONTEXT_SETTINGS, cls=DYMGroup)
@click.pass_context
def usage(ctx):
    """Retrieve resource usage related information
    """
    ctx.obj['client'] = ctx.obj['client'].usage


@usage.command('get')
@click.argument('resource-type')
@click.option('-d',
              '--date',
              help='Resource usage per day [defaults to yesterday]')
@click.pass_context
def get_usage(ctx, resource_type, date):
    """Retrieve usage statistics by day and resource type.

    Specifying Date will get you the usage for that day.
    Specifying Resource type will get you the usage for a particular resources
    by date.
    Omitting date will get you the usage for yesterday.
    """
    usage = ctx.obj['client']
    if date and resource_type:
        print(usage.get(resource_type=resource_type, date=date))
    elif resource_type:
        print(usage.get(resource_type=resource_type))
    elif date:
        print(usage.get(date=date))
    else:
        print(usage.get())


@_cloudhealth.group(context_settings=CLICK_CONTEXT_SETTINGS, cls=DYMGroup)
@click.pass_context
def reports(ctx):
    """Retrieve report related information
    """
    ctx.obj['client'] = ctx.obj['client'].reports


@reports.command('list')
@click.option('-t',
              '--topic',
              help='The topic to get the reports for')
@click.pass_context
def list_reports(ctx, topic):
    """List all reports

    Specifying a topic will get the reports only for that topic.
    """
    reports = ctx.obj['client']
    reports_list = reports.list(topic)
    for report in reports_list:
        print(report)


@reports.command('list-topics')
@click.pass_context
def list_topics(ctx):
    """List all topics
    """
    reports = ctx.obj['client']
    topics_list = reports.topics()
    for topic in topics_list:
        print(topic)


@reports.command(name='get')
@click.option('-i',
              '--id',
              default=None,
              help='The ID of the report')
@click.option('-t',
              '--topic',
              default=None,
              help='The topic of the report')
@click.option('-n',
              '--report-name',
              default=None,
              help='The name of the report')
@click.pass_context
def get_report(ctx, id, topic, report_name):
    """Retrieve a specific report
    """
    reports = ctx.obj['client']
    print(reports.get(id, topic, report_name))


@_cloudhealth.group(context_settings=CLICK_CONTEXT_SETTINGS, cls=DYMGroup)
@click.pass_context
def assets(ctx):
    """Retrieve assets related information
    """
    ctx.obj['client'] = ctx.obj['client'].assets


@assets.command(name='list')
@click.pass_context
def list_assets(ctx):
    """List all assets
    """
    assets = ctx.obj['client']
    assets_list = assets.list()
    for asset in assets_list:
        print(asset)


@assets.command(name='get')
@click.argument('asset-name')
@click.pass_context
def get_asset(ctx, object_name):
    """Retrieve a specific asset.
    """
    assets = ctx.obj['client']
    print(assets.get(object_name))


@_cloudhealth.group(context_settings=CLICK_CONTEXT_SETTINGS, cls=DYMGroup)
@click.pass_context
def accounts(ctx):
    """Retrieve accounts related information
    """
    ctx.obj['client'] = ctx.obj['client'].accounts


@accounts.command(name='list')
@click.option('-t',
              '--account-type',
              default='AWS-Account',
              help='The type to get the cost for [default: AWS-Account]')
@click.pass_context
def list_accounts(ctx, account_type):
    accounts = ctx.obj['client']
    accounts_list = accounts.list(account_type)
    for account in accounts_list:
        print(account)



########
# Copyright (c) 2013 GigaSpaces Technologies Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
#    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    * See the License for the specific language governing permissions and
#    * limitations under the License.

from setuptools import setup

setup(
    name='cloudhealth',
    version='0.1.1',
    author='Nir Cohen',
    author_email='nir36g@gmail.com',
    packages=['cloudhealth'],
    license='LICENSE',
    description='A REST Client for Cloudhealth',
    entry_points={
        'console_scripts': [
            'cloudhealth = cloudhealth.cli:_cloudhealth',
            'clh = cloudhealth.cli:_cloudhealth'
        ]
    },
    install_requires=[
        'click==6.6',
        'requests==2.7.0',
        'click_didyoumean==0.0.3',
    ]
)
